==> astk-serveur-1.5.2.1.tbz: aster/ASTK/ASTK_SERV/lib/as_utils.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
   Some utilities for Code_Aster
"""

__revision__ = "$Id: $"

__all__ = ['listsurcharge', 'vers4comp']

# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re

#-------------------------------------------------------------------------------
def _listsurcharge_u2mess(fsurch, lf):
   """Write the fortran subroutine "surchg.f".
      fsurch : subroutine filename
      lf     : list of modified files
   """
   header = """
      SUBROUTINE SURCHG(IFM)
C --- IFM CONSERVE POUR LA COMPATIBILITE
      INTEGER IFM
      INTEGER NBFILE
      CHARACTER*16  VALK(%(nbfile)s)"""
   fmt_line = "      VALK(%(indice)d) = '%(subroutine)s%(separ)s'"
   fmt_call = "      CALL U2MESK('A','SUPERVIS_40',%(nbfile)s,VALK)"
   footer = "      END"
   cont = []
   l_file = [os.path.basename(f) for f in lf]
   l_file.sort()
   nbfile = len(l_file)
   cont.append(header % { 'nbfile' : nbfile })
   npl = 6
   for i, file in enumerate(l_file):
      sep = ', '
      if i + 1 == len(l_file):
         sep = ''
      cont.append(fmt_line % { 'indice' : i + 1, 'subroutine' : file, 'separ' : sep })
   
   cont.append(fmt_call % { 'nbfile' : nbfile })
   cont.append(footer)
   
   open(fsurch, 'w').write(os.linesep.join(cont))
   #print os.linesep.join(cont)

#-------------------------------------------------------------------------------
def _listsurcharge_utmess(fsurch, lf):
   """Write the fortran subroutine "surchg.f".
      fsurch : subroutine filename
      lf     : list of modified files
   """
   cont = ["""      SUBROUTINE SURCHG(IFM)
      INTEGER IFM
      CALL UTMESS('A', 'SURCHG', 'VERSION SURCHARGEE PAR :')"""]
   templ = "      WRITE(IFM,'(3X,A)') '"
   sep = ', '
   lmax = 72
   len0 = len(templ)
   files = [os.path.basename(f) for f in lf]
   files.sort()
   while len(files)>0:
      lig = []
      clen = len0-len(sep)
      while len(files)>0 and clen+len(files[0])+len(sep) <= lmax-1:
         clen = clen+len(files[0])+len(sep)
         lig.append("%s" % files.pop(0))
      cont.append(templ + sep.join(lig) + "'")
      # si on n'a rien pu ajouter, il faut tronquer
      if clen == len0-len(sep):
         files[0] = files[0][:lmax-1-clen-len(sep)]
   cont.extend(['      END', ''])
   f = open(fsurch, 'w')
   f.write(os.linesep.join(cont))
   f.close()

#-------------------------------------------------------------------------------
def listsurcharge(version, fsurch, lf):
   """Aiguillage pour les versions avant U2MESS ( < 8.3.14)
   Exemple : listsurcharge('9.0.5', '/tmp/surchg.f', ['rout01.f', 'rout02.f'])
   """
   version = vers4comp(version)
   if version < ' 8. 3.14':
      func = _listsurcharge_utmess
   else:
      func = _listsurcharge_u2mess
   func(fsurch, lf)

#-------------------------------------------------------------------------------
def vers4comp(str_vers, n_indice=3):
   """A partir d'une version fournie sous forme d'une chaine (sparateur = '.'),
   retourne un chaine o chaque indice est crit sur 2 chiffres pour facilier
   les comparaisons (avec `n_indice` indices).
   "version for comparison" !
   """
   l_vers = str(str_vers).split('.')
   if len(l_vers) < n_indice:
      l_vers.extend(['0'] * (n_indice - len(l_vers)))
   vers = '.'.join(['%2s' % v for v in l_vers[:3]])
   return vers

#-------------------------------------------------------------------------------
def get_list(ftest):
   """Build list of the tests, ignore all after # or %  (characters comment).
   """
   iret = 0
   ltest = []
   try:
      line = open(ftest, 'r').read().splitlines()
   except (OSError, IOError):
      iret = 4
      return iret, ltest
   
   lfich = [t for t in [re.sub(' *[#%]+.*', '', l).strip() for l in line] if t != '']
   # delete twins (by keeping ordering)
   ltest = []
   for t in lfich:
      if not t in ltest:
         ltest.append(t)
   
   return iret, ltest

#-------------------------------------------------------------------------------
def getpara(fpara, platform=None):
   """Return a dict object by parsing the file 'fpara' written as :
   param_name1 value1 param_name2 value2...
   (parameters name should be in list_param).
   Be careful : tps_job is in seconds in fpara and in minutes in .export (LSF) !
   """
   list_param = ['mem_job', 'tps_job', 'ncpus', 'mem_aster']
   iret, err = 0, ''
   dico = {}
   for k in list_param:
      dico[k] = 0

   # ----- check that file exists
   if not os.path.exists(fpara):
      iret = 1
      err = _('file not found : %s') % fpara
      return iret, {}, err

   # ----- parsing
   f = open(fpara, 'r')
   for str0 in f:
      line = str0.split()
      while len(line)>0:
         w = line.pop(0)
         if w in list_param:
            dico[w] = line.pop(0)
   f.close()

   # ----- ncpus keep as string...
   # ----- convert time to int
   try:
      dico['tps_job'] = int(dico['tps_job'])
   except ValueError:
      iret = 1
      err = _('incorrect value for tps_job (%s) in %s') % (dico['tps_job'], fpara)
      return iret, {}, err

   # ----- delete 'Mo' in mem_job value
   try:
      dico['mem_job'] = int(dico['mem_job'].replace('Mo', ''))
   except ValueError:
      iret = 1
      err = _('incorrect value for mem_job (%s) in %s') % (dico['mem_job'], fpara)
      return iret, {}, err

   # ----- convert mem_aster to int
   try:
      dico['mem_aster'] = int(dico['mem_aster'])
   except ValueError:
      iret = 1
      err = _('incorrect value for mem_aster (%s) in %s') % (dico['mem_aster'], fpara)
      return iret, {}, err
   if dico['mem_aster'] == 0:
      dico['mem_aster'] = 100
   
   if platform is not None:
      if re.search('64$', platform):
         facW = 8
      else:
         facW = 4
      dico['memjeveux'] = dico['mem_job'] / facW * dico['mem_aster'] / 100.
   
   return iret, dico, err

==> astk-serveur-1.7.1.1.tbz: aster/ASTK/ASTK_SERV/lib/as_utils.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
   Some utilities for as_run.
"""

__revision__ = "$Id: as_utils.py 3656 2008-12-05 11:54:14Z courtois $"

# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re

#-------------------------------------------------------------------------------
def _listsurcharge_u2mess(fsurch, lf):
   """Write the fortran subroutine "surchg.f".
      fsurch : subroutine filename
      lf     : list of modified files
   """
   header = """
      SUBROUTINE SURCHG(IFM)
C --- IFM CONSERVE POUR LA COMPATIBILITE
      INTEGER IFM
      INTEGER NBFILE
      CHARACTER*16  VALK(%(nbfile)s)"""
   fmt_line = "      VALK(%(indice)d) = '%(subroutine)s%(separ)s'"
   fmt_call = "      CALL U2MESK('A','SUPERVIS_40',%(nbfile)s,VALK)"
   footer = "      END"
   cont = []
   l_file = [os.path.basename(f) for f in lf]
   l_file.sort()
   nbfile = len(l_file)
   cont.append(header % { 'nbfile' : max(1, nbfile) })
   npl = 6
   for i, file in enumerate(l_file):
      sep = ', '
      if i + 1 == len(l_file):
         sep = ''
      cont.append(fmt_line % { 'indice' : i + 1, 'subroutine' : file, 'separ' : sep })
   
   cont.append(fmt_call % { 'nbfile' : nbfile })
   cont.append(footer)
   
   open(fsurch, 'w').write(os.linesep.join(cont))
   #print os.linesep.join(cont)

#-------------------------------------------------------------------------------
def _listsurcharge_utmess(fsurch, lf):
   """Write the fortran subroutine "surchg.f".
      fsurch : subroutine filename
      lf     : list of modified files
   """
   cont = ["""      SUBROUTINE SURCHG(IFM)
      INTEGER IFM
      CALL UTMESS('A', 'SURCHG', 'VERSION SURCHARGEE PAR :')"""]
   templ = "      WRITE(IFM,'(3X,A)') '"
   sep = ', '
   lmax = 72
   len0 = len(templ)
   files = [os.path.basename(f) for f in lf]
   files.sort()
   while len(files)>0:
      lig = []
      clen = len0-len(sep)
      while len(files)>0 and clen+len(files[0])+len(sep) <= lmax-1:
         clen = clen+len(files[0])+len(sep)
         lig.append("%s" % files.pop(0))
      cont.append(templ + sep.join(lig) + "'")
      # si on n'a rien pu ajouter, il faut tronquer
      if clen == len0-len(sep):
         files[0] = files[0][:lmax-1-clen-len(sep)]
   cont.extend(['      END', ''])
   f = open(fsurch, 'w')
   f.write(os.linesep.join(cont))
   f.close()

#-------------------------------------------------------------------------------
def listsurcharge(version, fsurch, lf):
   """Aiguillage pour les versions avant U2MESS ( < 8.3.14)
   Exemple : listsurcharge('9.0.5', '/tmp/surchg.f', ['rout01.f', 'rout02.f'])
   """
   version = vers4comp(version)
   if version < ' 8. 3.14':
      func = _listsurcharge_utmess
   else:
      func = _listsurcharge_u2mess
   func(fsurch, lf)

#-------------------------------------------------------------------------------
def vers4comp(str_vers, n_indice=3):
   """A partir d'une version fournie sous forme d'une chaine (sparateur = '.'),
   retourne un chaine o chaque indice est crit sur 2 chiffres pour facilier
   les comparaisons (avec `n_indice` indices).
   "version for comparison" !
   """
   l_vers = str(str_vers).split('.')
   if len(l_vers) < n_indice:
      l_vers.extend(['0'] * (n_indice - len(l_vers)))
   vers = '.'.join(['%2s' % v for v in l_vers[:3]])
   return vers

#-------------------------------------------------------------------------------
def get_list(ftest, unique=True):
   """Build list of the tests, ignore all after # or %  (characters comment).
   """
   iret = 0
   ltest = []
   try:
      line = open(ftest, 'r').read().splitlines()
   except (OSError, IOError):
      iret = 4
      return iret, ltest
   
   lfich = [t for t in [re.sub(' *[#%]+.*', '', l).strip() for l in line] if t != '']
   if unique:
      # delete twins (by keeping ordering)
      ltest = []
      for t in lfich:
         if not t in ltest:
            ltest.append(t)
   else:
      ltest = lfich
   
   return iret, ltest

#-------------------------------------------------------------------------------
def getpara(fpara, platform=None):
   """Return a dict object by parsing the file 'fpara' written as :
   param_name1 value1 param_name2 value2...
   (parameters name should be in list_param).
   Be careful : tps_job is in seconds in fpara and in minutes in .export (LSF) !
   memjeveux in Mw, memjeveux_stat in MB...
   """
   list_param = ['mem_job', 'tps_job', 'mem_aster', 'memjeveux', 'memjeveux_stat',
                 'ncpus', 'mpi_nbnoeud', 'mpi_nbcpu',]
   iret, err = 0, ''
   dico = {}
   for k in list_param:
      dico[k] = 0

   # ----- check that file exists
   if not os.path.exists(fpara):
      iret = 1
      err = _('file not found : %s') % fpara
      return iret, {}, err

   # ----- parsing
   f = open(fpara, 'r')
   for str0 in f:
      line = str0.split()
      while len(line)>0:
         w = line.pop(0)
         if w in list_param:
            dico[w] = line.pop(0)
   f.close()

   # ----- ncpus keep as string...
   # ----- convert time to int
   try:
      dico['tps_job'] = int(float(dico['tps_job']))
   except ValueError:
      iret = 1
      err = _('incorrect value for tps_job (%s) in %s') % (dico['tps_job'], fpara)
      return iret, {}, err

   # ----- delete 'Mo' in mem_job value
   try:
      dico['mem_job'] = int(float(dico['mem_job'].replace('Mo', '')))
   except ValueError:
      iret = 1
      err = _('incorrect value for mem_job (%s) in %s') % (dico['mem_job'], fpara)
      return iret, {}, err

   # ----- delete 'Mo' in mem_job value
   try:
      dico['memjeveux_stat'] = int(float(dico['memjeveux_stat']))
   except ValueError:
      iret = 1
      err = _('incorrect value for memjeveux_stat (%s) in %s') % (dico['memjeveux_stat'], fpara)
      return iret, {}, err

   # ----- convert mem_aster to int
   try:
      dico['mem_aster'] = int(float(dico['mem_aster']))
   except ValueError:
      iret = 1
      err = _('incorrect value for mem_aster (%s) in %s') % (dico['mem_aster'], fpara)
      return iret, {}, err
   if dico['mem_aster'] == 0:
      dico['mem_aster'] = 100
   
   if platform is not None:
      if re.search('64$', platform):
         facW = 8
      else:
         facW = 4
      dico['memjeveux'] = dico['mem_job'] / facW * dico['mem_aster'] / 100.
   
   return iret, dico, err

#-------------------------------------------------------------------------------
def add_param(prof, dpara):
   """Add parameters and arguments to a profile.
   """
   prof.args['memjeveux']      = dpara['memjeveux']
   prof.args['tpmax']          = dpara['tps_job']
   if dpara['memjeveux_stat'] != 0:
      prof.args['memjeveux_stat'] = dpara['memjeveux_stat']
   new_para = {}
   for key, val in dpara.items():
      if prof.args.has_key(key):
         continue
      if key == 'tps_job':
         key = 'tpsjob'
         val = val / 60.
      elif key == 'mem_job':
         key = 'memjob'
         val = val * 1024
      if val != 0:
         new_para[key] = [val,]
   prof.param.update(new_para)
